#include <headers.h>


#ifdef CONFIG_NETWORK_ENABLED
#ifdef CONFIG_TELNET_ENABLED

TaskHandle_t TaskHandle_telnetTask;

// Telnet Port to Use
#define TELNET_TX_BUFFER		512
#define TELNET_RX_BUFFER 		256

char tx_buffer[TELNET_TX_BUFFER]={0};
char rx_bufer[TELNET_RX_BUFFER]={0};

volatile bool clientConnected = false;
int TelnetSock;
/* -----------------------------------------------------------------------------
  printft(const char *format, ...)

  Notes:  printft("Hello World\r\n");
 -----------------------------------------------------------------------------*/
void IRAM_ATTR printft(const char *format, ...)
{
	//printf(tx_buffer);
   
    // If telnet is connected then send it to there as well. 
    if (clientConnected)
    {
		va_list args;
		va_start(args, format);
   		vsnprintf(tx_buffer, TELNET_TX_BUFFER, format, args);
		va_end(args);		

       send(TelnetSock, tx_buffer, strlen(tx_buffer), 0);
    }
}
/* -----------------------------------------------------------------------------
  MyMainTask()

  Notes:  My Main Task, I.E. My version of loop but as its own xtask
 -----------------------------------------------------------------------------*/
void TaskTelnet(void *parameter)
{
	esp_log_level_set(TELNET_TAG, ESP_LOG_VERBOSE);
	
	struct sockaddr_in clientAddress;
	struct sockaddr_in serverAddress;
	int sizeRead;
	int sock, rc;

	clientConnected = false;

	// Create a socket that we will listen upon.
	sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (sock < 0)
	{
		ESP_LOGE(TELNET_TAG, "socket: %d %s", sock, strerror(errno));
		goto END;
	}

	// Bind our server socket to a port.
	serverAddress.sin_family = AF_INET;
	serverAddress.sin_addr.s_addr = htonl(INADDR_ANY);
	serverAddress.sin_port = htons(CONFIG_TELNET_PORT_NUMBER);
	rc = bind(sock, (struct sockaddr *)&serverAddress, sizeof(serverAddress));
	if (rc < 0)
	{
		ESP_LOGE(TELNET_TAG, "Error bind: %d %s", rc, strerror(errno));
		goto END;
	}

	// Flag the socket as listening for new connections.
	rc = listen(sock, 5);
	if (rc < 0)
	{
		ESP_LOGE(TELNET_TAG, "Error listen: %d %s", rc, strerror(errno));
		goto END;
	}

	for (;;)
	{
		ESP_LOGI(TELNET_TAG, "Telnet Server Listening on Port: %d", CONFIG_TELNET_PORT_NUMBER);

		// Listen for a new client connection.
		socklen_t clientAddressLength = sizeof(clientAddress);

		TelnetSock = accept(sock, (struct sockaddr *)&clientAddress, &clientAddressLength);
		if (TelnetSock < 0)
		{
			ESP_LOGE(TELNET_TAG, "Error accept: %d %s", TelnetSock, strerror(errno));
			goto END;
		}

		ESP_LOGI(TELNET_TAG, "Telnet Connection Established");
		clientConnected = true;


		// Loop reading data.
		do
		{
			sizeRead = recv(TelnetSock, rx_bufer, TELNET_RX_BUFFER, 0);

			if (sizeRead > 0)
			{
				// Slap a null on the end of it
				rx_bufer[sizeRead] = 0;

				ESP_LOGI(TELNET_TAG, "RX[%d]: %s", sizeRead, rx_bufer);
			}

			vTaskDelay(250 / portTICK_PERIOD_MS);

		} while (sizeRead >= 0);

		// Close the socket
		clientConnected = false;
		ESP_LOGI(TELNET_TAG, "Telenet Connection Closed");
		close(TelnetSock);
	}

END:
	ESP_LOGI(TELNET_TAG, "Terminating Telnet Task");
	vTaskDelete(NULL);
}

#endif	// CONFIG_TELNET_ENABLED
#endif // CONFIG_NETWORK_ENABLED